/*
    SpecRunner - Acceptance Test Driven Development Tool
    Copyright (C) 2011-2012  Thiago Santos

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package org.specrunner.dumper.impl;

import java.io.File;
import java.util.Map;

import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Nodes;

import org.specrunner.dumper.SourceDumperException;
import org.specrunner.result.IResultSet;
import org.specrunner.source.ISource;
import org.specrunner.source.SourceException;
import org.specrunner.util.UtilLog;

/**
 * Dump the central part of the result report (the 'center' element of the
 * framed version).
 * 
 * @author Thiago Santos
 * 
 */
public class SourceDumperCenter extends AbstractSourceDumperFile {

    @Override
    public void dump(ISource source, IResultSet result, Map<String, Object> model) throws SourceDumperException {
        set(source, result);
        String frameName = outputName;
        model.put(SourceDumperFrame.CENTER_FRAME, frameName);
        File output = new File(outputDirectory, frameName);
        try {
            addLinkToFrame(source.getDocument(), result, model);
            saveTo(source.getDocument(), output);
            System.out.println("Output (" + Thread.currentThread().getName() + "): " + output);
            if (UtilLog.LOG.isInfoEnabled()) {
                UtilLog.LOG.info("DEFAULT   SAVED TO " + output.getAbsolutePath());
            }
        } catch (SourceException e) {
            throw new SourceDumperException(e);
        }
    }

    protected void addLinkToFrame(Document document, IResultSet result, Map<String, Object> model) {
        Nodes nodes = document.query("//body");
        Element body = (Element) nodes.get(0);
        Element div = new Element("div");
        div.addAttribute(new Attribute("class", "sr_frame_link_div"));
        body.insertChild(div, 0);

        Element span = new Element("span");
        span.addAttribute(new Attribute("id", "linkFrame"));
        span.addAttribute(new Attribute("class", "sr_frame_link_span"));
        div.appendChild(span);

        Element link = new Element("a");
        span.appendChild(link);
        link.addAttribute(new Attribute("href", getFilePrefix() + "_frame.html"));
        link.addAttribute(new Attribute("title", "This is a framed report where errors are easier to find and test meta-information is available."));
        link.appendChild("Framed report");
        span.appendChild(" (in " + model.get("time") + " mls): ");
        span.appendChild(result.asNode());
        span.appendChild(" at " + model.get("date") + "");
    }
}
