/*
    SpecRunner - Acceptance Test Driven Development Tool
    Copyright (C) 2011-2012  Thiago Santos

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package org.specrunner.plugins.data;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import nu.xom.Node;
import nu.xom.Nodes;

import org.specrunner.context.IContext;
import org.specrunner.plugins.ENext;
import org.specrunner.plugins.PluginException;
import org.specrunner.plugins.impl.AbstractPluginScoped;
import org.specrunner.plugins.impl.UtilPlugin;
import org.specrunner.result.IResultSet;
import org.specrunner.result.Status;

/**
 * Replaces data list items by their values.
 * 
 * @author Thiago Santos
 * 
 */
public class PluginDataList extends AbstractPluginScoped {

    @Override
    public ENext doStart(IContext context, IResultSet result) throws PluginException {
        Node node = context.getNode();

        UtilPlugin.performChildren(node, context, result);

        final List<Pair> data = new LinkedList<Pair>();

        Nodes lis = node.query("child::li");

        for (int i = 0; i < lis.size(); i++) {
            Node li = lis.get(i);
            data.add(new Pair(li, li.getValue()));
        }

        IDataList<Pair> map = new IDataList<Pair>() {
            @Override
            public int getTotal() {
                return data.size();
            }

            @Override
            public Iterator<Pair> iterator() {
                return data.iterator();
            }
        };

        saveLocal(context, getName(), map);

        result.addResult(Status.SUCCESS, context.newBlock(node, this));

        return ENext.SKIP;
    }

    public static class Pair {
        private Node source;
        private Object data;

        public Pair(Node source, Object data) {
            this.source = source;
            this.data = data;
        }

        public Node getSource() {
            return source;
        }

        public Object getData() {
            return data;
        }
    }
}