/*
    SpecRunner - Acceptance Test Driven Development Tool
    Copyright (C) 2011-2012  Thiago Santos

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package org.specrunner.plugins.impl.factories;

import nu.xom.Element;
import nu.xom.Node;

import org.specrunner.context.IContext;
import org.specrunner.plugins.IPlugin;
import org.specrunner.plugins.PluginException;
import org.specrunner.plugins.impl.PluginNop;
import org.specrunner.plugins.impl.UtilPlugin;

/**
 * Creates a plugin by element. To bind an element add a '
 * <code>plugin_element.properties</code>' file and map the tag.
 * <p>
 * i.e. a map of <code>br=test.PluginNewLine</code> will perfome the plugin each
 * tag <code>&lt;br&gt;</code>.
 * 
 * @author Thiago Santos
 * 
 */
public class PluginFactoryElement extends PluginFactoryImpl {

    public PluginFactoryElement() {
        super("plugin_element.properties");
    }

    @Override
    public IPlugin newPlugin(Node node, IContext context) throws PluginException {
        initialize();
        if (node instanceof Element) {
            Element ele = (Element) node;
            String name = ele.getQualifiedName().toLowerCase();
            Class<? extends IPlugin> c = types.get(name);
            if (c != null) {
                return UtilPlugin.create(context, c, ele);
            }
        }
        return PluginNop.emptyPlugin();
    }
}